@@ -32,11 +32,16 @@ First you need to set up a generator for your kind of database connections
 by creating an instance of PersistentPg, passing the following parameters:
 
 	maxusage: the maximum number of reuses of a single connection
-		(the default of 0 or False means unlimited reuse)
+		(the default of 0 or None means unlimited reuse)
 		When this maximum usage number of the connection is reached,
 		the connection is automatically reset (closed and reopened).
 	setsession: An optional list of SQL commands that may serve to
 		prepare the session, e.g. ["set datestyle to german", ...]
+	closeable: if this is set to true, then closing connections will
+		be allowed, but by default this will be silently ignored
+	threadlocal: an optional class for representing thread-local data
+		that will be used instead of our Python implementation
+		(threading.local is faster, but cannot be used in all cases)
 
 	Additionally, you have to pass the parameters for the actual
 	PostgreSQL connection which are passed via PyGreSQL,
@@ -61,19 +66,23 @@ Closing a persistent connection with db.close() will be silently
 ignored since it would be reopened at the next usage anyway and
 contrary to the intent of having persistent connections. Instead,
 the connection will be automatically closed when the thread dies.
-You can change this behavior be setting persist._closeable to True.
+You can change this behavior be setting the closeable parameter.
+
+Note that by setting the threadlocal parameter to threading.local,
+getting connections may become a bit faster, but this may not work in
+all environments (for instance, mod_wsgi is known to cause problems
+since it clears the threading.local data between requests).
 
 
 Requirements:
 
-Minimum requirement: Python 2.2 and PyGreSQL 3.4.
-Recommended: Python 2.4.3 and PyGreSQL 3.8.
+Python >= 2.2, < 3.0, PyGreSQL >= 3.4.
 
 
 Ideas for improvement:
 
-* Add thread for monitoring and restarting bad or expired connections
-  (similar to DBConnectionPool/ResourcePool by Warren Smith).
+* Add a thread for monitoring, restarting (or closing) bad or expired
+  connections (similar to DBConnectionPool/ResourcePool by Warren Smith).
 * Optionally log usage, bad connections and exceeding of limits.
 
 
@@ -88,11 +97,12 @@ Licensed under the Open Software License version 2.1.
 
 """
 
-__version__ = '0.9.4'
-__revision__ = "$Rev: 6696 $"
-__date__ = "$Date: 2007-07-07 11:02:24 -0600 (Sat, 07 Jul 2007) $"
+__version__ = '1.0'
+__revision__ = "$Rev: 7680 $"
+__date__ = "$Date: 2008-11-29 07:55:36 -0700 (Sat, 29 Nov 2008) $"
 
 
+import ThreadingLocal
 from DBUtils.SteadyPg import SteadyPgConnection
 
 
@@ -104,32 +114,38 @@ class PersistentPg:
 
 	"""
 
-	def __init__(self, maxusage=0, setsession=None, *args, **kwargs):
+	version = __version__
+
+	def __init__(self, maxusage=None, setsession=None,
+			closeable=False, threadlocal=None, *args, **kwargs):
 		"""Set up the persistent PostgreSQL connection generator.
 
 		maxusage: maximum number of reuses of a single connection
-			(0 or False means unlimited reuse)
+			(0 or None means unlimited reuse)
 			When this maximum usage number of the connection is reached,
 			the connection is automatically reset (closed and reopened).
 		setsession: optional list of SQL commands that may serve to prepare
 			the session, e.g. ["set datestyle to ...", "set time zone ..."]
+		closeable: if this is set to true, then closing connections will
+			be allowed, but by default this will be silently ignored
+		threadlocal: an optional class for representing thread-local data
+			that will be used instead of our Python implementation
+			(threading.local is faster, but cannot be used in all cases)
 		args, kwargs: the parameters that shall be used to establish
 			the PostgreSQL connections using class PyGreSQL pg.DB()
 
-		Set the _closeable attribute to True or 1 to allow closing
-		connections. By default, this will be silently ignored.
-
 		"""
 		self._maxusage = maxusage
 		self._setsession = setsession
+		self._closeable = closeable
 		self._args, self._kwargs = args, kwargs
-		self._closeable = 0
-		self.thread = local()
+		self.thread = (threadlocal or ThreadingLocal.local)()
 
 	def steady_connection(self):
 		"""Get a steady, non-persistent PyGreSQL connection."""
 		return SteadyPgConnection(
-			self._maxusage, self._setsession, *self._args, **self._kwargs)
+			self._maxusage, self._setsession, self._closeable,
+			*self._args, **self._kwargs)
 
 	def connection(self):
 		"""Get a steady, persistent PyGreSQL connection."""
@@ -137,84 +153,5 @@ class PersistentPg:
 			con = self.thread.connection
 		except AttributeError:
 			con = self.steady_connection()
-			con._closeable = self._closeable
 			self.thread.connection = con
 		return con
-
-
-try: # import a class for representing thread-local objects
-	from threading import local
-except ImportError: # for Python < 2.4, use the following simple implementation
-	from threading import currentThread, enumerate, RLock
-	class _localbase(object):
-		__slots__ = '_local__key', '_local__args', '_local__lock'
-		def __new__(cls, *args, **kwargs):
-			self = object.__new__(cls)
-			key = '_local__key', 'thread.local.' + str(id(self))
-			object.__setattr__(self, '_local__key', key)
-			object.__setattr__(self, '_local__args', (args, kwargs))
-			object.__setattr__(self, '_local__lock', RLock())
-			if args or kwargs and (cls.__init__ is object.__init__):
-				raise TypeError("Initialization arguments are not supported")
-			d = object.__getattribute__(self, '__dict__')
-			currentThread().__dict__[key] = d
-			return self
-	def _patch(self):
-		key = object.__getattribute__(self, '_local__key')
-		d = currentThread().__dict__.get(key)
-		if d is None:
-			d = {}
-			currentThread().__dict__[key] = d
-			object.__setattr__(self, '__dict__', d)
-			cls = type(self)
-			if cls.__init__ is not object.__init__:
-				args, kwargs = object.__getattribute__(self, '_local__args')
-				cls.__init__(self, *args, **kwargs)
-		else:
-			object.__setattr__(self, '__dict__', d)
-	class local(_localbase):
-		def __getattribute__(self, name):
-			lock = object.__getattribute__(self, '_local__lock')
-			lock.acquire()
-			try:
-				_patch(self)
-				return object.__getattribute__(self, name)
-			finally:
-				lock.release()
-		def __setattr__(self, name, value):
-			lock = object.__getattribute__(self, '_local__lock')
-			lock.acquire()
-			try:
-				_patch(self)
-				return object.__setattr__(self, name, value)
-			finally:
-				lock.release()
-		def __delattr__(self, name):
-			lock = object.__getattribute__(self, '_local__lock')
-			lock.acquire()
-			try:
-				_patch(self)
-				return object.__delattr__(self, name)
-			finally:
-				lock.release()
-		def __del__():
-			threading_enumerate = enumerate
-			__getattribute__ = object.__getattribute__
-			def __del__(self):
-				try:
-					key = __getattribute__(self, '_local__key')
-					threads = list(threading_enumerate())
-				except Exception:
-					return
-				for thread in threads:
-					try:
-						__dict__ = thread.__dict__
-					except AttributeError:
-						continue
-					if key in __dict__:
-						try:
-							del __dict__[key]
-						except KeyError:
-							pass
-			return __del__
-		__del__ = __del__()
