@@ -32,22 +32,25 @@ an instance of PooledDB, passing the following parameters:
 	mincached: the initial number of idle connections in the pool
 		(the default of 0 means no connections are made at startup)
 	maxcached: the maximum number of idle connections in the pool
-		(the default value of 0 means unlimited pool size)
+		(the default value of 0 or None means unlimited pool size)
 	maxshared: maximum number of shared connections allowed
-		(the default value of 0 means all connections are dedicated)
+		(the default value of 0 or None means all connections are dedicated)
 		When this maximum number is reached, connections are
 		shared if they have been requested as shareable.
 	maxconnections: maximum number of connections generally allowed
-		(the default value of 0 means any number of connections)
+		(the default value of 0 or None means any number of connections)
 	blocking: determines behavior when exceeding the maximum
-		(the default of 0 or False means report an error; otherwise
-		block and wait until the number of connections decreases)
+		(if this is set to true, block and wait until the number of
+		connections decreases, but by default an error will be reported)
 	maxusage: maximum number of reuses of a single connection
-		(the default of 0 or False means unlimited reuse)
+		(the default of 0 or None means unlimited reuse)
 		When this maximum usage number of the connection is reached,
 		the connection is automatically reset (closed and reopened).
 	setsession: an optional list of SQL commands that may serve to
 		prepare the session, e.g. ["set datestyle to german", ...]
+	failures: an optional exception class or a tuple of exception classes
+		for which the connection failover mechanism shall be applied,
+		if the default (OperationalError, InternalError) is not adequate
 
 	The creator function or the connect function of the DB-API 2 compliant
 	database module specified as the creator will receive any additional
@@ -75,7 +78,11 @@ Please note that the connection may be shared with other threads
 by default if you set a non-zero maxshared parameter and the DB-API 2
 module allows this. If you want to have a dedicated connection, use:
 
-	db = pool.connection(0)
+	db = pool.connection(shareable=False)
+
+You can also use this to get a dedicated connection:
+
+	db = pool.dedicated_connection()
 
 If you don't need it any more, you should immediately return it to the
 pool with db.close(). You can get another connection in the same way.
@@ -98,8 +105,8 @@ connection object stays alive as long as you are using it, like that:
 
 Ideas for improvement:
 
-* Add thread for monitoring and restarting bad or expired connections
-  (similar to DBConnectionPool/ResourcePool by Warren Smith).
+* Add a thread for monitoring, restarting (or closing) bad or expired
+  connections (similar to DBConnectionPool/ResourcePool by Warren Smith).
 * Optionally log usage, bad connections and exceeding of limits.
 
 
@@ -114,15 +121,16 @@ Licensed under the Open Software License version 2.1.
 
 """
 
-__version__ = '0.9.4'
-__revision__ = "$Rev: 6696 $"
-__date__ = "$Date: 2007-07-07 11:02:24 -0600 (Sat, 07 Jul 2007) $"
+__version__ = '1.0'
+__revision__ = "$Rev: 7680 $"
+__date__ = "$Date: 2008-11-29 07:55:36 -0700 (Sat, 29 Nov 2008) $"
 
 
 from threading import Condition
 
 from DBUtils.SteadyDB import connect
 
+
 class PooledDBError(Exception):
 	"""General PooledDB error."""
 
@@ -144,11 +152,13 @@ class PooledDB:
 
 	"""
 
+	version = __version__
+
 	def __init__(self, creator,
-		mincached=0, maxcached=0,
-		maxshared=0, maxconnections=0, blocking=0,
-		maxusage=0, setsession=None,
-		*args, **kwargs):
+			mincached=0, maxcached=0,
+			maxshared=0, maxconnections=0, blocking=False,
+			maxusage=None, setsession=None, failures=None,
+			*args, **kwargs):
 		"""Set up the DB-API 2 connection pool.
 
 		creator: either an arbitrary function returning new DB-API 2
@@ -156,22 +166,25 @@ class PooledDB:
 		mincached: initial number of idle connections in the pool
 			(0 means no connections are made at startup)
 		maxcached: maximum number of idle connections in the pool
-			(0 means unlimited pool size)
+			(0 or None means unlimited pool size)
 		maxshared: maximum number of shared connections
-			(0 means all connections are dedicated)
+			(0 or None means all connections are dedicated)
 			When this maximum number is reached, connections are
 			shared if they have been requested as shareable.
 		maxconnections: maximum number of connections generally allowed
-			(0 means an arbitrary number of connections)
+			(0 or None means an arbitrary number of connections)
 		blocking: determines behavior when exceeding the maximum
-			(0 or False means report an error; otherwise
-			block and wait until the number of connections decreases)
+			(if this is set to true, block and wait until the number of
+			connections decreases, otherwise an error will be reported)
 		maxusage: maximum number of reuses of a single connection
-			(0 or False means unlimited reuse)
+			(0 or None means unlimited reuse)
 			When this maximum usage number of the connection is reached,
 			the connection is automatically reset (closed and reopened).
 		setsession: optional list of SQL commands that may serve to prepare
 			the session, e.g. ["set datestyle to ...", "set time zone ..."]
+		failures: an optional exception class or a tuple of exception classes
+			for which the connection failover mechanism shall be applied,
+			if the default (OperationalError, InternalError) is not adequate
 		args, kwargs: the parameters that shall be passed to the creator
 			function or the connection constructor of the DB-API 2 module
 
@@ -180,15 +193,25 @@ class PooledDB:
 			threadsafety = creator.threadsafety
 		except AttributeError:
 			try:
-				threadsafety = callable(creator.connect) and 0 or 2
+				if not callable(creator.connect):
+					raise AttributeError
 			except AttributeError:
 				threadsafety = 2
+			else:
+				threadsafety = 0
 		if not threadsafety:
 			raise NotSupportedError("Database module is not thread-safe.")
 		self._creator = creator
 		self._args, self._kwargs = args, kwargs
 		self._maxusage = maxusage
 		self._setsession = setsession
+		self._failures = failures
+		if mincached is None:
+			mincached = 0
+		if maxcached is None:
+			maxcached = 0
+		if maxconnections is None:
+			maxconnections = 0
 		if maxcached:
 			if maxcached < mincached:
 				maxcached = mincached
@@ -209,21 +232,24 @@ class PooledDB:
 		else:
 			self._maxconnections = 0
 		self._idle_cache = [] # the actual pool of idle connections
-		self._connections = 0
 		self._condition = Condition()
 		if not blocking:
 			def wait():
 				raise TooManyConnections
 			self._condition.wait = wait
+		self._connections = 0
 		# Establish an initial number of idle database connections:
-		[self.connection(0) for i in range(mincached)]
+		idle = [self.dedicated_connection() for i in range(mincached)]
+		while idle:
+			idle.pop().close()
 
 	def steady_connection(self):
 		"""Get a steady, unpooled DB-API 2 connection."""
 		return connect(self._creator,
-			self._maxusage, self._setsession, *self._args, **self._kwargs)
+			self._maxusage, self._setsession, self._failures, True,
+				*self._args, **self._kwargs)
 
-	def connection(self, shareable=1):
+	def connection(self, shareable=True):
 		""""Get a steady, cached DB-API 2 connection from the pool.
 
 		If shareable is set and the underlying DB-API 2 allows it,
@@ -233,8 +259,8 @@ class PooledDB:
 		if shareable and self._maxshared:
 			self._condition.acquire()
 			try:
-				while not self._shared_cache and self._maxconnections \
-					and self._connections >= self._maxconnections:
+				while (not self._shared_cache and self._maxconnections
+						and self._connections >= self._maxconnections):
 					self._condition.wait()
 				if len(self._shared_cache) < self._maxshared:
 					# shared cache is not full, get a dedicated connection
@@ -257,8 +283,8 @@ class PooledDB:
 		else: # try to get a dedicated connection
 			self._condition.acquire()
 			try:
-				while self._maxconnections \
-					and self._connections >= self._maxconnections:
+				while (self._maxconnections
+						and self._connections >= self._maxconnections):
 					self._condition.wait()
 				# connection limit not reached, get a dedicated connection
 				try: # first try to get it from the idle cache
@@ -271,6 +297,10 @@ class PooledDB:
 				self._condition.release()
 		return con
 
+	def dedicated_connection(self):
+		"""Alias for connection(shareable=False)."""
+		return self.connection(False)
+
 	def unshare(self, con):
 		"""Decrease the share of a connection in the shared cache."""
 		self._condition.acquire()
@@ -313,11 +343,18 @@ class PooledDB:
 		self._condition.acquire()
 		try:
 			while self._idle_cache: # close all idle connections
-				self._idle_cache.pop(0).close()
-				self._connections -= 1
+				con = self._idle_cache.pop(0)
+				try:
+					con.close()
+				except Exception:
+					pass
 			if self._maxshared: # close all shared connections
 				while self._shared_cache:
-					self._shared_cache.pop(0).con.close()
+					con = self._shared_cache.pop(0).con
+					try:
+						con.close()
+					except Exception:
+						pass
 					self._connections -= 1
 			self._condition.notifyAll()
 		finally:
@@ -325,8 +362,10 @@ class PooledDB:
 
 	def __del__(self):
 		"""Delete the pool."""
-		if hasattr(self, '_connections'):
+		try:
 			self.close()
+		except Exception:
+			pass
 
 
 # Auxiliary classes for pooled connections
@@ -341,6 +380,9 @@ class PooledDedicatedDBConnection:
 		con: the underlying SteadyDB connection
 
 		"""
+		# basic initialization to make finalizer work
+		self._con = None
+		# proper initialization of the connection
 		if not con.threadsafety():
 			raise NotSupportedError("Database module is not thread-safe.")
 		self._pool = pool
@@ -363,7 +405,10 @@ class PooledDedicatedDBConnection:
 
 	def __del__(self):
 		"""Delete the pooled connection."""
-		self.close()
+		try:
+			self.close()
+		except Exception:
+			pass
 
 
 class SharedDBConnection:
@@ -401,6 +446,9 @@ class PooledSharedDBConnection:
 		con: the underlying SharedDBConnection
 
 		"""
+		# basic initialization to make finalizer work
+		self._con = None
+		# proper initialization of the connection
 		con = shared_con.con
 		if not con.threadsafety() > 1:
 			raise NotSupportedError("Database connection is not thread-safe.")
@@ -425,4 +473,7 @@ class PooledSharedDBConnection:
 
 	def __del__(self):
 		"""Delete the pooled connection."""
-		self.close()
+		try:
+			self.close()
+		except Exception:
+			pass
