@@ -73,9 +73,9 @@ Licensed under the Open Software License version 2.1.
 
 """
 
-__version__ = '0.9.4'
-__revision__ = "$Rev: 6696 $"
-__date__ = "$Date: 2007-07-07 11:02:24 -0600 (Sat, 07 Jul 2007) $"
+__version__ = '1.0'
+__revision__ = "$Rev: 7680 $"
+__date__ = "$Date: 2008-11-29 07:55:36 -0700 (Sat, 29 Nov 2008) $"
 
 
 class PooledDBError(Exception):
@@ -121,6 +121,8 @@ class PooledDB:
 
 	"""
 
+	version = __version__
+
 	def __init__(self, dbapi, maxconnections, *args, **kwargs):
 		"""Set up the database connection pool.
 
