@@ -11,9 +11,9 @@ Copyright and credit info:
 
 """
 
-__version__ = '0.9.4'
-__revision__ = "$Rev: 6696 $"
-__date__ = "$Date: 2007-07-07 11:02:24 -0600 (Sat, 07 Jul 2007) $"
+__version__ = '1.0'
+__revision__ = "$Rev: 7680 $"
+__date__ = "$Date: 2008-11-29 07:55:36 -0700 (Sat, 29 Nov 2008) $"
 
 
 import sys
@@ -28,9 +28,11 @@ from DBUtils.PooledDB import PooledDB
 class TestPooledDB(unittest.TestCase):
 
 	def test00_CheckVersion(self):
-		TestPooledDBVersion = __version__
+		from DBUtils import __version__ as DBUtilsVersion
+		self.assertEqual(DBUtilsVersion, __version__)
 		from DBUtils.PooledDB import __version__ as PooledDBVersion
-		self.assertEqual(PooledDBVersion, TestPooledDBVersion)
+		self.assertEqual(PooledDBVersion, __version__)
+		self.assertEqual(PooledDB.version, __version__)
 
 	def test01_NoThreadsafety(self):
 		from DBUtils.PooledDB import NotSupportedError
@@ -54,7 +56,7 @@ class TestPooledDB(unittest.TestCase):
 		for threadsafety in (1, 2):
 			dbapi.threadsafety = threadsafety
 			shareable = threadsafety > 1
-			pool = PooledDB(dbapi, 1, 1, 1, 0, 0, 0, None,
+			pool = PooledDB(dbapi, 1, 1, 1, 0, False, None, None, None,
 				'PooledDBTestDB', user='PooledDBTestUser')
 			self.assert_(hasattr(pool, '_idle_cache'))
 			self.assertEqual(len(pool._idle_cache), 1)
@@ -64,7 +66,7 @@ class TestPooledDB(unittest.TestCase):
 			else:
 				self.assert_(not hasattr(pool, '_shared_cache'))
 			self.assert_(hasattr(pool, '_maxusage'))
-			self.assertEqual(pool._maxusage, 0)
+			self.assertEqual(pool._maxusage, None)
 			self.assert_(hasattr(pool, '_setsession'))
 			self.assert_(pool._setsession is None)
 			con = pool._idle_cache[0]
@@ -118,7 +120,26 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(db_con.session,
 				['rollback', 'sessiontest'])
 			pool = PooledDB(dbapi, 1, 1, 1)
-			db = pool.connection(0)
+			self.assertEqual(len(pool._idle_cache), 1)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 0)
+			db = pool.connection()
+			self.assertEqual(len(pool._idle_cache), 0)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 1)
+			db.close()
+			self.assertEqual(len(pool._idle_cache), 1)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 0)
+			db = pool.connection(True)
+			self.assertEqual(len(pool._idle_cache), 0)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 1)
+			db.close()
+			self.assertEqual(len(pool._idle_cache), 1)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 0)
+			db = pool.connection(False)
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
@@ -126,7 +147,23 @@ class TestPooledDB(unittest.TestCase):
 			db_con = db._con._con
 			self.assert_(db_con.database is None)
 			self.assert_(db_con.user is None)
-			pool = PooledDB(dbapi, 0, 0, 0, 0, 0, 3, ('set datestyle',),)
+			db.close()
+			self.assertEqual(len(pool._idle_cache), 1)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 0)
+			db = pool.dedicated_connection()
+			self.assertEqual(len(pool._idle_cache), 0)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 0)
+			self.assertEqual(db._usage, 0)
+			db_con = db._con._con
+			self.assert_(db_con.database is None)
+			self.assert_(db_con.user is None)
+			db.close()
+			self.assertEqual(len(pool._idle_cache), 1)
+			if shareable:
+				self.assertEqual(len(pool._shared_cache), 0)
+			pool = PooledDB(dbapi, 0, 0, 0, 0, False, 3, ('set datestyle',))
 			self.assertEqual(pool._maxusage, 3)
 			self.assertEqual(pool._setsession, ('set datestyle',))
 			con = pool.connection()._con
@@ -137,7 +174,7 @@ class TestPooledDB(unittest.TestCase):
 		for threadsafety in (1, 2):
 			dbapi.threadsafety = threadsafety
 			shareable = threadsafety > 1
-			pool = PooledDB(dbapi, 0, 1, 1, 0, 0, 0, None,
+			pool = PooledDB(dbapi, 0, 1, 1, 0, False, None, None, None,
 				'PooledDBTestDB', user='PooledDBTestUser')
 			self.assert_(hasattr(pool, '_idle_cache'))
 			self.assertEqual(len(pool._idle_cache), 0)
@@ -202,7 +239,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(pool._idle_cache[0]._con, db_con)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(db._con, con)
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
@@ -302,7 +339,7 @@ class TestPooledDB(unittest.TestCase):
 				self.assertNotEqual(db4._con, db1._con)
 				self.assertNotEqual(db4._con, db2._con)
 				self.assertNotEqual(db4._con, db3._con)
-			db5 = pool.connection(0)
+			db5 = pool.connection(False)
 			self.assertNotEqual(db5._con, db1._con)
 			self.assertNotEqual(db5._con, db2._con)
 			self.assertNotEqual(db5._con, db3._con)
@@ -323,7 +360,7 @@ class TestPooledDB(unittest.TestCase):
 				self.assertEqual(len(pool._idle_cache), 0)
 			pool = PooledDB(dbapi, 0, 0, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
-			db1 = pool.connection(0)
+			db1 = pool.connection(False)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
 			db2 = pool.connection()
@@ -402,7 +439,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(len(pool._idle_cache), 5)
 			pool = PooledDB(dbapi, 1, 2, 3)
 			self.assertEqual(len(pool._idle_cache), 1)
-			cache = [pool.connection(0) for i in range(4)]
+			cache = [pool.connection(False) for i in range(4)]
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
@@ -418,7 +455,7 @@ class TestPooledDB(unittest.TestCase):
 				self.assertEqual(len(pool._shared_cache), 0)
 			pool = PooledDB(dbapi, 1, 3, 2)
 			self.assertEqual(len(pool._idle_cache), 1)
-			cache = [pool.connection(0) for i in range(4)]
+			cache = [pool.connection(False) for i in range(4)]
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
@@ -459,7 +496,7 @@ class TestPooledDB(unittest.TestCase):
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 1)
 			pool = PooledDB(dbapi, 0, 0, 7)
-			cache = [pool.connection(0) for i in range(3)]
+			cache = [pool.connection(False) for i in range(3)]
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
@@ -475,7 +512,7 @@ class TestPooledDB(unittest.TestCase):
 			pool = PooledDB(dbapi, 5, 5, 5)
 			self.assertEqual(len(pool._idle_cache), 5)
 			for i in range(15):
-				db = pool.connection(0)
+				db = pool.connection(False)
 				db.cursor().execute('select test')
 				db.close()
 			self.assertEqual(len(pool._idle_cache), 5)
@@ -559,7 +596,7 @@ class TestPooledDB(unittest.TestCase):
 			dbapi.threadsafety = threadsafety
 			pool = PooledDB(dbapi, 0, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(len(pool._idle_cache), 0)
 			self.assertEqual(db._con._con.open_cursors, 0)
 			cursor = db.cursor()
@@ -571,7 +608,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(db._con._con.open_cursors, 0)
 			del db
 			self.assertEqual(len(pool._idle_cache), 1)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(len(pool._idle_cache), 0)
 			self.assertEqual(db._con._con.open_cursors, 0)
 			cursor = db.cursor()
@@ -599,7 +636,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(len(pool._idle_cache), 1)
 			cache = []
 			for i in range(3):
-				cache.append(pool.connection(0))
+				cache.append(pool.connection(False))
 			self.assertEqual(pool._connections, 3)
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
@@ -617,14 +654,14 @@ class TestPooledDB(unittest.TestCase):
 			if shareable:
 				self.assertEqual(pool._connections, 2)
 				self.assertEqual(len(pool._shared_cache), 2)
-				cache.append(pool.connection(0))
+				cache.append(pool.connection(False))
 				self.assertEqual(pool._connections, 3)
 				self.assertEqual(len(pool._shared_cache), 2)
 			else:
 				self.assertEqual(pool._connections, 3)
 			self.assertRaises(TooManyConnections, pool.connection, 0)
 			if shareable:
-				cache.append(pool.connection(1))
+				cache.append(pool.connection(True))
 				self.assertEqual(pool._connections, 3)
 			else:
 				self.assertRaises(TooManyConnections, pool.connection)
@@ -635,7 +672,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(pool._maxconnections, 1)
 			self.assertEqual(pool._connections, 0)
 			self.assertEqual(len(pool._idle_cache), 0)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(pool._connections, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
@@ -658,7 +695,7 @@ class TestPooledDB(unittest.TestCase):
 				self.assertRaises(TooManyConnections, pool.connection)
 			self.assertRaises(TooManyConnections, pool.connection, 0)
 			if shareable:
-				cache.append(pool.connection(1))
+				cache.append(pool.connection(True))
 				self.assertEqual(pool._connections, 1)
 				self.assertEqual(len(pool._shared_cache), 1)
 				self.assertEqual(pool._shared_cache[0].shared, 3)
@@ -669,7 +706,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(len(pool._idle_cache), 1)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(pool._connections, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
 			del db
@@ -680,28 +717,28 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(pool._connections, 0)
 			self.assertEqual(len(pool._idle_cache), 1)
 			cache = []
-			cache.append(pool.connection(0))
+			cache.append(pool.connection(False))
 			self.assertEqual(pool._connections, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
-			cache.append(pool.connection(0))
+			cache.append(pool.connection(False))
 			self.assertEqual(pool._connections, 2)
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
 			self.assertRaises(TooManyConnections, pool.connection, 0)
 			self.assertRaises(TooManyConnections, pool.connection)
-			pool = PooledDB(dbapi, 4, 3, 2, 1, 0)
+			pool = PooledDB(dbapi, 4, 3, 2, 1, False)
 			self.assertEqual(pool._maxconnections, 4)
 			self.assertEqual(pool._connections, 0)
 			self.assertEqual(len(pool._idle_cache), 4)
 			cache = []
 			for i in range(4):
-				cache.append(pool.connection(0))
+				cache.append(pool.connection(False))
 			self.assertEqual(pool._connections, 4)
 			self.assertEqual(len(pool._idle_cache), 0)
 			self.assertRaises(TooManyConnections, pool.connection, 0)
 			self.assertRaises(TooManyConnections, pool.connection)
-			pool = PooledDB(dbapi, 1, 2, 3, 4, 0)
+			pool = PooledDB(dbapi, 1, 2, 3, 4, False)
 			self.assertEqual(pool._maxconnections, 4)
 			self.assertEqual(pool._connections, 0)
 			self.assertEqual(len(pool._idle_cache), 1)
@@ -713,18 +750,18 @@ class TestPooledDB(unittest.TestCase):
 				self.assertEqual(len(pool._shared_cache), 3)
 				cache.append(pool.connection())
 				self.assertEqual(pool._connections, 3)
-				cache.append(pool.connection(0))
+				cache.append(pool.connection(False))
 				self.assertEqual(pool._connections, 4)
 			else:
 				self.assertEqual(pool._connections, 4)
 				self.assertRaises(TooManyConnections, pool.connection)
 			self.assertRaises(TooManyConnections, pool.connection, 0)
-			pool = PooledDB(dbapi, 0, 0, 3, 3, 0)
+			pool = PooledDB(dbapi, 0, 0, 3, 3, False)
 			self.assertEqual(pool._maxconnections, 3)
 			self.assertEqual(pool._connections, 0)
 			cache = []
 			for i in range(3):
-				cache.append(pool.connection(0))
+				cache.append(pool.connection(False))
 			self.assertEqual(pool._connections, 3)
 			self.assertRaises(TooManyConnections, pool.connection, 0)
 			self.assertRaises(TooManyConnections, pool.connection, 1)
@@ -745,30 +782,31 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(pool._connections, 0)
 			cache = []
 			for i in range(10):
-				cache.append(pool.connection(0))
+				cache.append(pool.connection(False))
 				cache.append(pool.connection())
 			if shareable:
 				self.assertEqual(pool._connections, 13)
 				self.assertEqual(len(pool._shared_cache), 3)
 			else:
 				self.assertEqual(pool._connections, 20)
-			pool = PooledDB(dbapi, 1, 1, 1, 1, 1)
+			pool = PooledDB(dbapi, 1, 1, 1, 1, True)
 			self.assertEqual(pool._maxconnections, 1)
 			self.assertEqual(pool._connections, 0)
 			self.assertEqual(len(pool._idle_cache), 1)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(pool._connections, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
 			def connection():
 				db = pool.connection()
-				cursor=db.cursor()
+				cursor = db.cursor()
 				cursor.execute('set thread')
 				cursor.close()
 				db.close()
 			from threading import Thread
-			Thread(target=connection).start()
-			from time import sleep
-			sleep(0.1)
+			thread = Thread(target=connection)
+			thread.start()
+			thread.join(0.1)
+			self.assert_(thread.isAlive())
 			self.assertEqual(pool._connections, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
 			if shareable:
@@ -776,12 +814,13 @@ class TestPooledDB(unittest.TestCase):
 			session = db._con._con.session
 			self.assertEqual(session, ['rollback'])
 			del db
-			sleep(0.1)
+			thread.join(0.1)
+			self.assert_(not thread.isAlive())
 			self.assertEqual(pool._connections, 0)
 			self.assertEqual(len(pool._idle_cache), 1)
 			if shareable:
 				self.assertEqual(len(pool._shared_cache), 0)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(pool._connections, 1)
 			self.assertEqual(len(pool._idle_cache), 0)
 			self.assertEqual(session, ['rollback',
@@ -791,10 +830,10 @@ class TestPooledDB(unittest.TestCase):
 		for threadsafety in (1, 2):
 			dbapi.threadsafety = threadsafety
 			for maxusage in (0, 3, 7):
-				pool = PooledDB(dbapi, 0, 0, 0, 1, 0, maxusage)
+				pool = PooledDB(dbapi, 0, 0, 0, 1, False, maxusage)
 				self.assertEqual(pool._maxusage, maxusage)
 				self.assertEqual(len(pool._idle_cache), 0)
-				db = pool.connection(0)
+				db = pool.connection(False)
 				self.assertEqual(db._con._maxusage, maxusage)
 				self.assertEqual(len(pool._idle_cache), 0)
 				self.assertEqual(db._con._con.open_cursors, 0)
@@ -826,9 +865,9 @@ class TestPooledDB(unittest.TestCase):
 		for threadsafety in (1, 2):
 			dbapi.threadsafety = threadsafety
 			setsession = ('set time zone', 'set datestyle')
-			pool = PooledDB(dbapi, 0, 0, 0, 1, 0, 0, setsession)
+			pool = PooledDB(dbapi, 0, 0, 0, 1, False, None, setsession)
 			self.assertEqual(pool._setsession, setsession)
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(db._setsession_sql, setsession)
 			self.assertEqual(db._con._con.session,
 				['time zone', 'datestyle'])
@@ -840,7 +879,7 @@ class TestPooledDB(unittest.TestCase):
 			self.assertEqual(db._con._con.session,
 				['time zone', 'datestyle', 'test1'])
 			db.close()
-			db = pool.connection(0)
+			db = pool.connection(False)
 			self.assertEqual(db._setsession_sql, setsession)
 			self.assertEqual(db._con._con.session,
 				['time zone', 'datestyle', 'test1', 'rollback'])
@@ -903,27 +942,27 @@ class TestPooledDB(unittest.TestCase):
 			else:
 				self.assertNotEqual(db1._con, db2._con)
 			del db1
-			db1 = pool.connection(0)
+			db1 = pool.connection(False)
 			self.assertNotEqual(db1, db2)
 			self.assertNotEqual(db1._con, db2._con)
 
 	def test16_ThreeThreadsTwoConnections(self):
 		for threadsafety in (1, 2):
 			dbapi.threadsafety = threadsafety
-			pool = PooledDB(dbapi, 2, 2, 0, 2, 1)
+			pool = PooledDB(dbapi, 2, 2, 0, 2, True)
 			from Queue import Queue, Empty
 			queue = Queue(3)
 			def connection():
 				try:
-					queue.put(pool.connection(), 1, 0.1)
+					queue.put(pool.connection(), 1, 1)
 				except Exception:
 					queue.put(pool.connection(), 1)
 			from threading import Thread
 			for i in range(3):
 				Thread(target=connection).start()
 			try:
-				db1 = queue.get(1, 0.1)
-				db2 = queue.get(1, 0.1)
+				db1 = queue.get(1, 1)
+				db2 = queue.get(1, 1)
 			except TypeError:
 				db1 = queue.get(1)
 				db2 = queue.get(1)
@@ -937,15 +976,15 @@ class TestPooledDB(unittest.TestCase):
 				self.assertRaises(Empty, queue.get, 0)
 			del db1
 			try:
-				db1 = queue.get(1, 0.1)
+				db1 = queue.get(1, 1)
 			except TypeError:
 				db1 = queue.get(1)
 			self.assertNotEqual(db1, db2)
 			self.assertNotEqual(db1._con, db2._con)
 			self.assertEqual(db1._con, db1_con)
-			pool = PooledDB(dbapi, 2, 2, 1, 2, 1)
-			db1 = pool.connection(0)
-			db2 = pool.connection(0)
+			pool = PooledDB(dbapi, 2, 2, 1, 2, True)
+			db1 = pool.connection(False)
+			db2 = pool.connection(False)
 			self.assertNotEqual(db1, db2)
 			db1_con = db1._con
 			db2_con = db2._con
@@ -957,7 +996,7 @@ class TestPooledDB(unittest.TestCase):
 				self.assertRaises(Empty, queue.get, 0)
 			del db1
 			try:
-				db1 = queue.get(1, 0.1)
+				db1 = queue.get(1, 1)
 			except TypeError:
 				db1 = queue.get(1)
 			self.assertNotEqual(db1, db2)
