@@ -11,9 +11,9 @@ Copyright and credit info:
 
 """
 
-__version__ = '0.9.4'
-__revision__ = "$Rev: 6696 $"
-__date__ = "$Date: 2007-07-07 11:02:24 -0600 (Sat, 07 Jul 2007) $"
+__version__ = '1.0'
+__revision__ = "$Rev: 7680 $"
+__date__ = "$Date: 2008-11-29 07:55:36 -0700 (Sat, 29 Nov 2008) $"
 
 
 import sys
@@ -28,17 +28,19 @@ from DBUtils.PooledPg import PooledPg
 class TestPooledPg(unittest.TestCase):
 
 	def test0_CheckVersion(self):
-		TestPooledPgVersion = __version__
+		from DBUtils import __version__ as DBUtilsVersion
+		self.assertEqual(DBUtilsVersion, __version__)
 		from DBUtils.PooledPg import __version__ as PooledPgVersion
-		self.assertEqual(PooledPgVersion, TestPooledPgVersion)
+		self.assertEqual(PooledPgVersion, __version__)
+		self.assertEqual(PooledPg.version, __version__)
 
 	def test1_CreateConnection(self):
-		pool = PooledPg(1, 1, 0, 0, 0, None,
+		pool = PooledPg(1, 1, 0, False, None, None,
 			'PooledPgTestDB', user='PooledPgTestUser')
 		self.assert_(hasattr(pool, '_cache'))
 		self.assertEqual(pool._cache.qsize(), 1)
 		self.assert_(hasattr(pool, '_maxusage'))
-		self.assertEqual(pool._maxusage, 0)
+		self.assertEqual(pool._maxusage, None)
 		self.assert_(hasattr(pool, '_setsession'))
 		self.assert_(pool._setsession is None)
 		db_con = pool._cache.get(0)
@@ -70,7 +72,7 @@ class TestPooledPg(unittest.TestCase):
 		self.assert_(db.user is None)
 		self.assert_(hasattr(db, 'num_queries'))
 		self.assertEqual(db.num_queries, 0)
-		pool = PooledPg(0, 0, 0, 0, 3, ('set datestyle',),)
+		pool = PooledPg(0, 0, 0, False, 3, ('set datestyle',),)
 		self.assertEqual(pool._maxusage, 3)
 		self.assertEqual(pool._setsession, ('set datestyle',))
 		db = pool.connection()
@@ -78,7 +80,7 @@ class TestPooledPg(unittest.TestCase):
 		self.assertEqual(db._setsession_sql, ('set datestyle',))
 
 	def test2_CloseConnection(self):
-		pool = PooledPg(0, 1, 0, 0, 0, None,
+		pool = PooledPg(0, 1, 0, False, None, None,
 			'PooledPgTestDB', user='PooledPgTestUser')
 		db = pool.connection()
 		self.assert_(hasattr(db, '_con'))
@@ -167,7 +169,7 @@ class TestPooledPg(unittest.TestCase):
 			cache.append(pool.connection())
 		self.assertEqual(pool._cache.qsize(), 0)
 		self.assertRaises(TooManyConnections, pool.connection)
-		pool = PooledPg(0, 1, 1, 0)
+		pool = PooledPg(0, 1, 1, False)
 		self.assertEqual(pool._blocking, 0)
 		self.assertEqual(pool._cache.qsize(), 0)
 		db = pool.connection()
@@ -181,14 +183,14 @@ class TestPooledPg(unittest.TestCase):
 		cache.append(pool.connection())
 		self.assertEqual(pool._cache.qsize(), 0)
 		self.assertRaises(TooManyConnections, pool.connection)
-		pool = PooledPg(3, 2, 1, 0)
+		pool = PooledPg(3, 2, 1, False)
 		self.assertEqual(pool._cache.qsize(), 3)
 		cache = []
 		for i in range(3):
 			cache.append(pool.connection())
 		self.assertEqual(pool._cache.qsize(), 0)
 		self.assertRaises(TooManyConnections, pool.connection)
-		pool = PooledPg(1, 1, 1, 1)
+		pool = PooledPg(1, 1, 1, True)
 		self.assertEqual(pool._blocking, 1)
 		self.assertEqual(pool._cache.qsize(), 1)
 		db = pool.connection()
@@ -196,14 +198,16 @@ class TestPooledPg(unittest.TestCase):
 		def connection():
 			pool.connection().query('set thread')
 		from threading import Thread
-		Thread(target=connection).start()
-		from time import sleep
-		sleep(0.1)
+		thread = Thread(target=connection)
+		thread.start()
+		thread.join(0.1)
+		self.assert_(thread.isAlive())
 		self.assertEqual(pool._cache.qsize(), 0)
 		session = db._con.session
 		self.assertEqual(session, [])
 		del db
-		sleep(0.1)
+		thread.join(0.1)
+		self.assert_(not thread.isAlive())
 		self.assertEqual(pool._cache.qsize(), 1)
 		db = pool.connection()
 		self.assertEqual(pool._cache.qsize(), 0)
@@ -233,20 +237,20 @@ class TestPooledPg(unittest.TestCase):
 		self.assertEqual(db2.num_queries, 8)
 
 	def test6_ThreeThreadsTwoConnections(self):
-		pool = PooledPg(2, 2, 2, 1)
+		pool = PooledPg(2, 2, 2, True)
 		from Queue import Queue, Empty
 		queue = Queue(3)
 		def connection():
 			try:
-				queue.put(pool.connection(), 1, 0.1)
+				queue.put(pool.connection(), 1, 1)
 			except TypeError:
 				queue.put(pool.connection(), 1)
 		from threading import Thread
 		for i in range(3):
 			Thread(target=connection).start()
 		try:
-			db1 = queue.get(1, 0.1)
-			db2 = queue.get(1, 0.1)
+			db1 = queue.get(1, 1)
+			db2 = queue.get(1, 1)
 		except TypeError:
 			db1 = queue.get(1)
 			db2 = queue.get(1)
@@ -260,7 +264,7 @@ class TestPooledPg(unittest.TestCase):
 			self.assertRaises(Empty, queue.get, 0)
 		del db1
 		try:
-			db1 = queue.get(1, 0.1)
+			db1 = queue.get(1, 1)
 		except TypeError:
 			db1 = queue.get(1)
 		self.assertNotEqual(db1, db2)
